@@ -110,7 +110,7 @@ class Init(Subcommand):
             meta = None
 
         feedstock_directory = args.feedstock_directory.format(package=argparse.Namespace(name=meta.name()))
-        msg = 'Initial commit of the {} feedstock.'.format(meta.name())
+        msg = 'Initial feedstock commit with conda-smithy {}.'.format(__version__)
 
         try:
             generate_feedstock_content(feedstock_directory, args.recipe_directory, meta)
@@ -229,8 +229,8 @@ class Regenerate(Subcommand):
                         print("")
                     else:
                         print(
-                            "You can commit the changes with:\n\n"
-                            "    git commit -m 'MNT: Re-rendered with conda-smithy %s'\n" % __version__
+                            'You can commit the changes with:\n\n'
+                            '    git commit -m "MNT: Re-rendered with conda-smithy %s"\n' % __version__
                         )
                     print("These changes need to be pushed to github!\n")
                 else:
