@@ -11,6 +11,33 @@ import warnings
 
 import conda.api
 import conda.config
+
+try:
+    # Try conda's API in newer 4.2.x and 4.3.x.
+    from conda.exports import (
+        DEFAULT_CHANNELS_UNIX,
+        DEFAULT_CHANNELS_WIN,
+    )
+except ImportError:
+    try:
+        # Fallback for old versions of 4.2.x and 4.3.x.
+        from conda.base.constants import (
+            DEFAULT_CHANNELS_UNIX,
+            DEFAULT_CHANNELS_WIN,
+        )
+    except ImportError:
+        # Fallback for very old conda (e.g. 4.1.x).
+        DEFAULT_CHANNELS_UNIX = (
+            'https://repo.continuum.io/pkgs/free',
+            'https://repo.continuum.io/pkgs/pro',
+        )
+
+        DEFAULT_CHANNELS_WIN = (
+            'https://repo.continuum.io/pkgs/free',
+            'https://repo.continuum.io/pkgs/pro',
+            'https://repo.continuum.io/pkgs/msys2',
+        )
+
 import conda_build.metadata
 try:
     import conda_build.api
@@ -23,8 +50,7 @@ from conda_build_all.resolved_distribution import ResolvedDistribution
 from jinja2 import Environment, FileSystemLoader
 
 from conda_smithy.feedstock_io import (
-    get_mode_file,
-    set_mode_file,
+    set_exe_file,
     write_file,
     remove_file,
     copy_file,
@@ -45,7 +71,11 @@ def render_run_docker_build(jinja_env, forge_config, forge_dir):
     meta = forge_config['package']
     with fudge_subdir('linux-64', build_config=meta_config(meta)):
         meta.parse_again()
-        matrix = compute_build_matrix(meta, forge_config.get('matrix'))
+        matrix = compute_build_matrix(
+            meta,
+            forge_config.get('matrix'),
+            forge_config.get('channels', {}).get('sources', tuple())
+        )
         cases_not_skipped = []
         for case in matrix:
             pkgs, vars = split_case(case)
@@ -70,6 +100,24 @@ def render_run_docker_build(jinja_env, forge_config, forge_dir):
         matrix = prepare_matrix_for_env_vars(matrix)
         forge_config = update_matrix(forge_config, matrix)
 
+        build_setup = ""
+
+        # If the recipe supplies its own conda-forge-build-setup script,
+        # we use it instead of the global one.
+        cfbs_fpath = os.path.join(forge_dir, 'recipe',
+                                  'run_conda_forge_build_setup_linux')
+        if os.path.exists(cfbs_fpath):
+            build_setup += textwrap.dedent("""\
+                # Overriding global conda-forge-build-setup with local copy.
+                source /recipe_root/run_conda_forge_build_setup_linux
+
+            """)
+        else:
+            build_setup += textwrap.dedent("""\
+                source run_conda_forge_build_setup
+
+            """)
+
         # If there is a "yum_requirements.txt" file in the recipe, we honour it.
         yum_requirements_fpath = os.path.join(forge_dir, 'recipe',
                                               'yum_requirements.txt')
@@ -81,7 +129,8 @@ def render_run_docker_build(jinja_env, forge_config, forge_dir):
                 raise ValueError("No yum requirements enabled in the "
                                  "yum_requirements.txt, please remove the file "
                                  "or add some.")
-            build_setup = textwrap.dedent("""\
+            build_setup += textwrap.dedent("""\
+
                 # Install the yum requirements defined canonically in the
                 # "recipe/yum_requirements.txt" file. After updating that file,
                 # run "conda smithy rerender" and this line be updated
@@ -90,7 +139,19 @@ def render_run_docker_build(jinja_env, forge_config, forge_dir):
 
 
             """.format(' '.join(requirements)))
-            forge_config['build_setup'] = build_setup
+
+        forge_config['build_setup'] = build_setup
+
+        # If the recipe supplies its own conda-forge-build-setup upload script,
+        # we use it instead of the global one.
+        upload_fpath = os.path.join(forge_dir, 'recipe',
+                                    'upload_or_check_non_existence.py')
+        if os.path.exists(upload_fpath):
+            forge_config['upload_script'] = (
+                "/recipe_root/upload_or_check_non_existence.py"
+            )
+        else:
+            forge_config['upload_script'] = "upload_or_check_non_existence"
 
         # TODO: Conda has a convenience for accessing nested yaml content.
         templates = forge_config.get('templates', {})
@@ -108,18 +169,18 @@ def render_run_docker_build(jinja_env, forge_config, forge_dir):
             os.path.join(forge_dir, 'ci_support', 'checkout_merge_commit.sh'),
         ]
         for each_target_fname in target_fnames:
-            mode = get_mode_file(each_target_fname)
-            set_mode_file(
-                each_target_fname,
-                mode | stat.S_IXOTH | stat.S_IXGRP | stat.S_IXUSR
-            )
+            set_exe_file(each_target_fname, True)
 
 
 def render_circle(jinja_env, forge_config, forge_dir):
     meta = forge_config['package']
     with fudge_subdir('linux-64', build_config=meta_config(meta)):
         meta.parse_again()
-        matrix = compute_build_matrix(meta, forge_config.get('matrix'))
+        matrix = compute_build_matrix(
+            meta,
+            forge_config.get('matrix'),
+            forge_config.get('channels', {}).get('sources', tuple())
+        )
 
         cases_not_skipped = []
         for case in matrix:
@@ -172,7 +233,11 @@ def render_travis(jinja_env, forge_config, forge_dir):
     meta = forge_config['package']
     with fudge_subdir('osx-64', build_config=meta_config(meta)):
         meta.parse_again()
-        matrix = compute_build_matrix(meta, forge_config.get('matrix'))
+        matrix = compute_build_matrix(
+            meta,
+            forge_config.get('matrix'),
+            forge_config.get('channels', {}).get('sources', tuple())
+        )
 
         cases_not_skipped = []
         for case in matrix:
@@ -193,6 +258,41 @@ def render_travis(jinja_env, forge_config, forge_dir):
         forge_config["travis"]["enabled"] = True
         matrix = prepare_matrix_for_env_vars(matrix)
         forge_config = update_matrix(forge_config, matrix)
+
+        build_setup = ""
+
+        # If the recipe supplies its own conda-forge-build-setup script,
+        # we use it instead of the global one.
+        cfbs_fpath = os.path.join(forge_dir, 'recipe',
+                                  'run_conda_forge_build_setup_osx')
+        if os.path.exists(cfbs_fpath):
+            build_setup += textwrap.dedent("""\
+                # Overriding global conda-forge-build-setup with local copy.
+                source {recipe_dir}/run_conda_forge_build_setup_osx
+            """.format(recipe_dir=forge_config["recipe_dir"]))
+        else:
+            build_setup += textwrap.dedent("""\
+                source run_conda_forge_build_setup
+            """)
+
+        build_setup = build_setup.strip()
+        build_setup = build_setup.replace("\n", "\n      ")
+
+        forge_config['build_setup'] = build_setup
+
+        # If the recipe supplies its own conda-forge-build-setup upload script,
+        # we use it instead of the global one.
+        upload_fpath = os.path.join(forge_dir, 'recipe',
+                                    'upload_or_check_non_existence.py')
+        if os.path.exists(upload_fpath):
+            forge_config['upload_script'] = (
+                "{recipe_dir}/upload_or_check_non_existence.py".format(
+                    recipe_dir=forge_config["recipe_dir"]
+                )
+            )
+        else:
+            forge_config['upload_script'] = "upload_or_check_non_existence"
+
         template = jinja_env.get_template('travis.yml.tmpl')
         with write_file(target_fname) as fh:
             fh.write(template.render(**forge_config))
@@ -269,7 +369,11 @@ def render_appveyor(jinja_env, forge_config, forge_dir):
     for platform, arch in [['win-32', 'x86'], ['win-64', 'x64']]:
         with fudge_subdir(platform, build_config=meta_config(meta)):
             meta.parse_again()
-            matrix = compute_build_matrix(meta, forge_config.get('matrix'))
+            matrix = compute_build_matrix(
+                meta,
+                forge_config.get('matrix'),
+                forge_config.get('channels', {}).get('sources', tuple())
+            )
 
             cases_not_skipped = []
             for case in matrix:
@@ -300,13 +404,14 @@ def render_appveyor(jinja_env, forge_config, forge_dir):
         for case in old_matrix:
             case = odict(case)
 
+            # Use Python 2.7 as a fallback when no Python version is set.
+            case["CONDA_PY"] = case.get("CONDA_PY", "27")
+
             # Set `root`'s `python` version.
             case["CONDA_INSTALL_LOCN"] = "C:\\\\Miniconda"
             if case.get("CONDA_PY") == "27":
                 case["CONDA_INSTALL_LOCN"] += ""
-            elif case.get("CONDA_PY") == "34":
-                case["CONDA_INSTALL_LOCN"] += "3"
-            elif case.get("CONDA_PY") == "35":
+            elif case.get("CONDA_PY") in ("35", "36"):
                 case["CONDA_INSTALL_LOCN"] += "35"
 
             # Set architecture.
@@ -319,6 +424,43 @@ def render_appveyor(jinja_env, forge_config, forge_dir):
         del old_matrix
 
         forge_config = update_matrix(forge_config, matrix)
+
+        build_setup = ""
+
+        # If the recipe supplies its own conda-forge-build-setup script,
+        # we use it instead of the global one.
+        cfbs_fpath = os.path.join(forge_dir, 'recipe',
+                                  'run_conda_forge_build_setup_osx')
+        if os.path.exists(cfbs_fpath):
+            build_setup += textwrap.dedent("""\
+                # Overriding global conda-forge-build-setup with local copy.
+                {recipe_dir}\\run_conda_forge_build_setup_win
+            """.format(recipe_dir=forge_config["recipe_dir"]))
+        else:
+            build_setup += textwrap.dedent("""\
+
+                run_conda_forge_build_setup
+            """)
+
+        build_setup = build_setup.rstrip()
+        build_setup = build_setup.replace("\n", "\n    - cmd: ")
+        build_setup = build_setup.lstrip()
+
+        forge_config['build_setup'] = build_setup
+
+        # If the recipe supplies its own conda-forge-build-setup upload script,
+        # we use it instead of the global one.
+        upload_fpath = os.path.join(forge_dir, 'recipe',
+                                    'upload_or_check_non_existence.py')
+        if os.path.exists(upload_fpath):
+            forge_config['upload_script'] = (
+                "{recipe_dir}\\upload_or_check_non_existence".format(
+                    recipe_dir=forge_config["recipe_dir"]
+                )
+            )
+        else:
+            forge_config['upload_script'] = "upload_or_check_non_existence"
+
         template = jinja_env.get_template('appveyor.yml.tmpl')
         with write_file(target_fname) as fh:
             fh.write(template.render(**forge_config))
@@ -399,10 +541,29 @@ def meta_of_feedstock(forge_dir, config=None):
     return meta
 
 
-def compute_build_matrix(meta, existing_matrix=None):
-    index = conda.api.get_index()
+def compute_build_matrix(meta, existing_matrix=None, channel_sources=tuple()):
+    channel_sources = tuple(channel_sources)
+
+    # Override what `defaults` means depending on the platform used.
+    try:
+        i = channel_sources.index("defaults")
+
+        defaults = DEFAULT_CHANNELS_UNIX
+        if meta_config(meta).subdir.startswith("win"):
+            defaults = DEFAULT_CHANNELS_WIN
+
+        channel_sources = (
+            channel_sources[:i] +
+            defaults +
+            channel_sources[i+1:]
+        )
+    except ValueError:
+        pass
+
+    index = conda.api.get_index(channel_urls=channel_sources,
+                                platform=meta_config(meta).subdir)
     mtx = special_case_version_matrix(meta, index)
-    mtx = list(filter_cases(mtx, ['python >=2.7,<3|>=3.4', 'numpy >=1.10']))
+    mtx = list(filter_cases(mtx, ['python >=2.7,<3|>=3.5', 'numpy >=1.11']))
     if existing_matrix:
         mtx = [tuple(mtx_case) + tuple(MatrixCaseEnvVar(*item) for item in case)
                for case in sorted(existing_matrix)
@@ -428,7 +589,8 @@ def main(forge_file_directory):
               'travis': {},
               'circle': {},
               'appveyor': {},
-              'channels': {'sources': ['conda-forge'], 'targets': [['conda-forge', 'main']]},
+              'channels': {'sources': ['conda-forge', 'defaults'],
+                           'targets': [['conda-forge', 'main']]},
               'github': {'user_or_org': 'conda-forge', 'repo_name': ''},
               'recipe_dir': recipe_dir}
     forge_dir = os.path.abspath(forge_file_directory)
